// fun3.h - Apstraktna klasa
//      funkcija i klasa delegata.

#ifndef _fun3_h_
#define _fun3_h_

#include <iostream>
#include <utility>
using std::ostream, std::move;

namespace Izvodi {
  class Delegat;  // Deklaracija klase delegata.

  class Fun {                               // DEFINICIJA KLASE FUNKCIJA:
  public:
    virtual ~Fun() {}                       // Virtuelan destruktor.
    virtual double operator()(double x) const =0; // Vrednost funkcije.
    virtual Delegat izvod() const =0;       // Izvod funkcije.
    virtual Fun* kopija() const& =0;        // Kopija funkcije kopiranjem.
    virtual Fun* kopija() && =0;            // Kopija funkcije premeštanjem.
 
  private:
    virtual void pisi(ostream& it) const =0;  // Pisanje funkcije.
    friend ostream& operator<<(ostream& it, const Fun& f)
      { f.pisi(it); return it; }
  }; // class Fun

  class Delegat: public Fun {                 // DEFINICIJA KLASE DELEGATA:
    Fun* f;                                   // Sadržana funkcija.
    void pisi(ostream& it) const override { it << *f; } // Pisanje delegata.
  public:
    Delegat(Fun* f): f{f} {}                         // Stvaranje delegata.
    Delegat(const Delegat& d) { f = d.f->kopija(); } // Kopirajući konstr.
    Delegat(Delegat&& d) { f = d.f; d.f = nullptr; } // Premeštajući konstr
    ~Delegat() { delete f; }                         // Uništavanje deleg.
    Delegat& operator=(const Delegat& d) {           // Kopirajuća dodela
      if (this != &d) { delete f; f = d.f->kopija(); }     // vrednosti.
      return *this;
    }
    Delegat& operator=(Delegat&& d) {                // Premeštajuća dodela
      if (this != &d) { delete f; f = d.f; d.f = nullptr;} // vrednosti.
      return *this;
    }
    double operator()(double x) const override       // Vrednost delegata.
      { return (*f)(x); }
    Delegat izvod() const override                   // Izvod delegata.
      { return f->izvod(); }
    Delegat* kopija() const& override       // Kopija delegata kopiranjem.
      { return new Delegat(*this); }
    Delegat* kopija() && override           // Kopija delegata premeštanjem.
      { return new Delegat(move(*this)); }
  }; // class Delegat
} // namespace Izvodi

#endif
