// inventar.h - Klasa inventara.

#include "zapis.h"
#include <iostream>
#include <string>
using std::ostream;

class Inventar {
  Zapis** niz;                                  // Niz pokazivača na zapise.
  int kap, duz;                                 // Kapacitet i dužina niza.
  void kopiraj(const Inventar& inv);            // Kopiranje u objekat.
  void premesti(Inventar& inv) {                // Premeštanje u objekat.
    niz = inv.niz; kap = inv.kap; duz = inv.duz;
    inv.niz = nullptr;
  }
  void brisi();                                 // Oslobađanje memorije.
public:
  explicit Inventar(int k=10)                   // Stvaranje objekta.
    : niz{new Zapis*[k]}, kap{k}, duz{0} {}
  Inventar(const Inventar& inv)                 // Kopirajući konstruktor.
    { kopiraj(inv); }
  Inventar(Inventar&& inv)                      // Premeštajući konstruktor.
    { premesti(inv); }
  ~Inventar() { brisi(); }                         // Destruktor.
  Inventar& operator=(const Inventar& inv) {       // Kopirajuća dodela
    if (this != &inv) { brisi(); kopiraj(inv); }   //   vrednosti.
    return *this;
  }
  Inventar& operator=(Inventar&& inv) {            // Premeštajuća dodela
    if (this != &inv) { brisi(); premesti(inv); }  //   vrednosti.
    return *this;
  }
  int duzina() const { return duz; }               // Dužina niza.
                                                   // Dohvatanje zapisa:
  Zapis* operator[](string naziv);                 // - iz promenljivog,
  const Zapis* operator[](string naziv) const      // - iz nepromenljivog
    {return const_cast<Inventar&>(*this)[naziv]; } // objekta.
  bool operator+=(Zapis* z) {                      // Dodavanje zapisa.
    if (duz==kap || (*this)[z->naziv()]) return false;
    niz[duz++] = z; return true;
  }
                                                      // Iteratori:
  Zapis** begin() { return niz; }                     // - za promenljive
  Zapis** end() { return niz + duz; }                 //   inventare,
  const Zapis*const*const begin() const {return niz;} // - za nepromenljive
  const Zapis*const*const end() const {return niz + duz;}  //   inventare.
  double vrednost() const;                       // Vrednost svih artikala.
  friend ostream& operator<<(ostream& it, const Inventar& inv); // Pisanje.
};
