// niz4.h – Generička klasa nizova.

#ifndef _niz4_h_
#define _niz4_h_

#include <iostream>
using std::ostream;

class G_indeks {};          // KLASA GREŠAKA: Nedozvoljen indeks.
inline ostream& operator<<(ostream& it, const G_indeks&)
  { return it << "*** Nedozvoljen indeks! ***"; }

class G_prazan {};          // KLASA GREŠAKA: Niz je prazan.
inline ostream& operator<<(ostream& it, const G_prazan&)
  { return it << "*** Niz je prazan! ***"; }

template <typename T>                          // GENERIČKA KLASA NIZOVA:
  class Niz {
    T* niz;                                    // Elementi niza.
    int kap, duz;                              // Kapacitet i dužina niza.
    void kopiraj(const Niz& n);                // Kopiranje u niz.
    void premesti(Niz& n) {                    // Premeštanje u niz.
      niz = n.niz; kap = n.kap; duz = n.duz;
      n.niz = nullptr;
    }
    void brisi() { delete [] niz; }            // Oslobađanje memorije.
  public:
    explicit Niz(int k=10)                     // Stvaranje praznog niza.
      : niz{new T[k]}, kap{k}, duz{0} {}
    Niz(const Niz& n) { kopiraj(n); }          // Kopirajući konstruktor.
    Niz(Niz&& n) { premesti(n); }              // Premeštajući konstruktor.
    ~Niz() { delete [] niz; }                  // Destruktor.
    Niz& operator=(const Niz& n) {                    // Kopirajuća dodela
      if (this != &n) { delete [] niz; kopiraj(n); }  //   vrednosti.
      return *this;
    }
    Niz& operator=(Niz&& n) {                         // Premeštajuća dodela
      if (this != &n) { delete [] niz; premesti(n); } //   vrednosti.
      return *this;
    }
    int vel() const { return duz; }            // Dužina niza.
    Niz& dodaj(const T& t);                    // Dodavanje na kraj.
    T uzmi();                                  // Uzimanje s kraja.
    T& operator[](int i) {                     // Dohvatanje elementa
      if (i<0 || i>=duz) throw G_indeks();     // - promenljivog niza,
      return niz[i];
    }
    const T& operator[](int i) const {         // - nepromenljivog niza.
      if (i<0 || i>=duz) throw G_indeks();
      return niz[i];
    }
    Niz& isprazni() { duz = 0; return *this; } // Pražnjenje niza.
  }; // class Niz<T>

template <typename T>                          // METODE KLASE NIZOVA:
  void Niz<T>::kopiraj(const Niz& n) {         // Kopiranje u objekat.
    niz = new T [kap = n.kap];
    duz = n.duz;
    for (int i=0; i<duz; i++)
      niz[i] = n.niz[i];
  }
template <typename T>
  Niz<T>& Niz<T>::dodaj(const T& t) {          // Dodavanje na kraj.
    if (duz == kap) {
      T* pom = new T [kap+=5];
      for (int i=0; i<duz; i++) pom[i] = niz[i];
      delete [] niz; niz = pom;
    }
    niz[duz++] = t;
    return *this;
  }

template <typename T>  
  T Niz<T>::uzmi() {                           // Uzimanje s kraja.
    if (!duz) throw G_prazan();
    T t = niz[--duz];
    if (kap-duz>10) {
      T* pom = new T [kap-=5];
      for (int i=0; i<duz; i++) pom[i] = niz[i];
      delete [] niz; niz = pom;
    }
    return t;
  }

#endif
