// niz5.h - Generička klasa nizova.

#ifndef _niz5_h_
#define _niz5_h_

#include <iostream>
using std::ostream;

class G_niz_pun {};                 // KLASA GREŠAKA: Niz je prepunjen.
inline ostream& operator<<(ostream& it, const G_niz_pun&)
  { return it << "*** Niz je pun! ***"; }

class G_indeks {};                  // KLASA GREŠAKA: Indeks izvan opsega.
inline ostream& operator<<(ostream& it, const G_indeks&)
  { return it << "*** Nedozvoljen indeks! ***"; }

template <typename T>                      // GENERIČKA KLASA NIZOVA:
class Niz {
  T* niz;                                  // Elementi niza.
  int kap, duz;                            // Kapacitet i dužina niza.
  void kopiraj(const Niz& n);              // Kopiranje u niz.
  void premesti(Niz n) {                   // Premeštanje u niz.
    niz = n.niz; kap = n.kap; duz = n.duz;
    n.niz = nullptr;
  }
  void brisi() { delete [] niz; }          // Oslobađanje memorije.
public:
  explicit Niz(int kap=20)                 // Stvaranje praznog niza.
    : niz{new T[kap]}, kap{kap}, duz{0} {}
  Niz(const Niz& n) { kopiraj(n); }        // Kopirajući konstruktor.
  Niz(Niz&& n) { premesti(n); }            // Premeštajući konstruktor.
  ~Niz() { brisi(); }                      // Destruktor.
  Niz& operator=(const Niz& n){            // Kopirajuća dodela vrednosti.
    if (this!=&n) { brisi(); kopiraj(n); }
    return *this;
  }
  Niz& operator=(Niz&& n){                 // Premeštajuća dodela vrednosti.
    if (this!=&n) { brisi(); premesti(n); }
    return *this;
  }
  int duzina() const { return duz; }       // Dužina niza.
  Niz& operator+=(const T& e) {            // Dodavanje elementa.
    if (duz == kap) throw G_niz_pun();
    niz[duz++] = e;
    return *this;
  }
  T& operator[](int i) {                   // Pristup elementu
    if (i<0 || i>=duz) throw G_indeks();   // - promenljivog niza,
    return niz[i];
  }
  const T& operator[](int i) const {       // - nepromenljivog niza.
    if (i<0 || i>=duz) throw G_indeks();
    return niz[i];
  }
};

template <typename T>                      // METODA KLASE NIZOVA:
void Niz<T>::kopiraj(const Niz& n) {       // Kopiranje u niz.
  niz = new T [kap = n.kap];
  duz = n.duz;
  for (int i=0; i<duz; i++) niz[i] = n.niz[i];
}

#endif
