// poli1.h - Klasa polinoma.

#include <iostream>
using std::istream, std::ostream;

class Poli {
  int n; double* a;          // Red polinoma i pokazivač na koeficijente.
  using CP = const Poli;     // Nepromenljiv polinom.
  void kopiraj(const double* a, int n);    // Kopiranje u polinom.
  void premesti(Poli& p)                   // Premeštanje u polinom.
    { n = p.n; a = p.a; p.a = nullptr; }
  void brisi() { delete [] a; a=nullptr; n=-1; }  // Oslobađane memorije.
public:                                           // Konstruktori:
  Poli(): n(-1), a(nullptr) {}                    // - prazan polinom,
  Poli(const double* a, int n) { kopiraj(a, n); } // - kopiranjem niza,
  Poli(CP& p) { kopiraj(p.a, p.n); }              // - kopiranjem polinoma,
  Poli(Poli&& p) { premesti(p); }                 // - premeštanjem polin.
  ~Poli() { brisi(); }                            // Destruktor.
                                                  // Dodela vrednosti:
  Poli& operator=(CP& p) {                        // - kopiranjem,
    if (this != &p) { brisi(); kopiraj(p.a, p.n); }
    return *this;
  }
  Poli& operator=(Poli&& p) {                      // - premeštanjem.
    if (this != &p) { brisi(); premesti(p); }
    return *this;
  }
  void niz(double* a, int& n) const;       // Smeštanje polinoma u niz.
  int red() const { return n; }            // Red polinoma.
  double operator()(double x) const;       // Vrednost polinoma.
  double& operator[](int ind);             // Pristupanje koeficijentu.
  const double& operator[](int ind) const;
  friend Poli operator+(CP& p1, CP& p2);                  // p1 +  p2
  friend Poli operator-(CP& p1, CP& p2);                  // p1 -  p2
  friend Poli operator*(CP& p1, CP& p2);                  // p1 *  p2
  Poli& operator+=(CP& p2) { return *this = *this + p2; } // p1 += p2
  Poli& operator-=(CP& p2) { return *this = *this - p2; } // p1 -= p2
  Poli& operator*=(CP& p2) { return *this = *this * p2; } // p1 *= p2
  friend istream& operator>>(istream& ut, Poli& p);  // Čitanje polinoma.
  friend ostream& operator<<(ostream& it, CP& p);    // Pisanje polinoma.
private:
  enum class Greska { G_RED, G_IND };                // Šifre grešaka.
  [[noreturn]] static void greska(Greska g);         // Obrada greške.
};
