// r_file.h - Klasa relativnih binarnih datoteka.

#include <fstream>
#include <string>
#include <iostream>
using std::fstream, std::ios, std::ostream,
      std::string;

class R_file : fstream {
public:
  class Greska {         // KLASA GREŠAKA.
  public:
    enum Gre { OK ,      // Šifre grešaka:
               DUZ,      // - neispravna dužina zapisa,
               BRO,      // - neispravan redni broj zapisa,
               VEL,      // - datoteka nije ceo broj zapisa,
               ZAT,      // - datoteka nije otvorena,
               IMA,      //   datoteka već postoji,
               NEMA,     //   datoteka ne postoji,
               OTV,      // - nije uspelo otvaranje,
               POZ,      // - nije uspelo pozicioniranje,
               PIS,      // - nije uspelo upisivanje,
               CIT,      // - nije uspelo čitanje,
               ZAP       // - zapis ne postoji.
             };
  private:
    Gre gre;                    // Šifra greške.
    static const string por[];  // Tekstualni opisi grešaka.
  public:
    Greska(Gre g) { gre = g; }
    friend ostream& operator<<(ostream& it, const Greska& g);
  };
                                            // ČLANOVI GLAVNE KLASE:
private:
  long duz_zap, bro_zap, tek_zap, tek_poz;  // bro_zap==-1 - nije otvorena.
  void otvori(const char* ime, long duz=0); // Otvaranje datoteke (duz>0
                                            //   - stvaranje nove).
public:
  R_file(): fstream(), bro_zap{-1} {}       // Inic. bez otvaranja datoteke.
  R_file(const char* ime, long duz=0)       // Inic. s otvaranjem datoteke.
    : fstream() { otvori(ime, duz); }
  ~R_file() { close(); }                    // Uništavanje objekta.
  R_file& open(const char* ime, long duz=0) // Otvaranje datoteke.
    { otvori(ime, duz); return *this; }
  bool open() { return bro_zap >= 0; }       // Da li je datoteka otvorena?
  R_file& write(const void* niz, long zap=0);// Pisanje zap (zap==0 - sekv).
  R_file& read(void* niz, long zap=0);       // Čitanje zap (zap==0 - sekv).
  R_file& close();                                  // Zatvaranje datoteke.
  long recn() { return bro_zap<0 ? -1 : bro_zap; } // Broj zapisa u datoteci
  long recl() { return bro_zap<0 ? -1 : duz_zap; } // Dužina zapisa.
};
