// skladiste.h - Generička klasa skladišta stvari.

#ifndef _skladiste_h_
#define _skladiste_h_

#include <iostream>
using std::ostream;

class G_puno {};                     // KLASA GREŠAKA: Skladište je puno.
inline ostream& operator<<(ostream& it, const G_puno&)
  { return it << "*** Skladiste je puno! ***"; }

class G_prazno {};                   // KLASA GREŠAKA: Skladište je prazno.
inline ostream& operator<<(ostream& it, const G_prazno&)
  { return it << "*** Skladiste je prazno! ***"; }

template <typename T>               // GENERIČKA KLASA SKLADIŠTA:
class Skladiste {
  T* niz;                           // Niz stvari.
  int kap, duz;                     // Kapacitet i popunjenost niza.
  int prvi, posl;                   // Indeks prve i poslednje stvari.
  void kopiraj(const Skladiste& s); // Kopiranje u skladište.
  void premesti(Skladiste s) {      // Premeštanje u skladište.
    niz = s.niz; kap = s.kap; duz = s.duz;
    prvi = s.prvi; posl = s.posl;
    s.niz = nullptr;
  }
  void brisi() { delete [] niz; }   // Oslobađanje memorije.
public:
  explicit Skladiste(int kap=10)    // Stvaranje praznog skladišta.
    : niz{new T[kap]}, kap{kap}, duz{0}, prvi{0}, posl{0} {}
  Skladiste(const Skladiste& s) { kopiraj(s); } // Kopirajući konstruktor.
  Skladiste(Skladiste&& s) { premesti(s); }     // Premeštajući konstruktor.
  ~Skladiste() { brisi(); }                     // Destruktor.
  Skladiste& operator=(const Skladiste& s) {    // Kopirajuća dodela
    if (this != &s) { brisi(); kopiraj(s); }    //   vrednosti.
    return *this;
  }
  Skladiste& operator=(Skladiste&& s) {          // Premeštajuća dodela
    if (this != &s) { brisi(); premesti(s); }    //   vrednosti.
    return *this;
  }
  void stavi(const T& t) {                       // Stavljanje u skladište.
    if (duz == kap) throw G_puno();
    niz[posl++] = t;
    if (posl == kap) posl = 0;
    duz++;
  }
  T uzmi() {                                     // Uzimanje iz skladišta.
    if (duz == 0) throw G_prazno();
    T t = niz[prvi++];
    if (prvi == kap) prvi = 0;
    duz --;
    return t;
  }
};

template <typename T>                            // METODA KLASE SKLADIŠTA:
void Skladiste<T>::kopiraj(const Skladiste& s) { // Kopiranje u skladište.
  niz = new T [kap = s.kap];
  for (int i=0; i<kap; i++) niz[i] = s.niz[i];
  prvi = s.prvi; posl = s.posl; duz = s.duz;
}

#endif
