// stablo.h - Definicija klase uređenih binarnih stabala.

#include <iostream>
#include <iomanip>
#include <cstdlib>
using std::cin, std::cout, std::endl, std::setw;

class Stablo {
  struct Cvor {                             // ČVOR STABLA:
    int broj;                               // - sadržaj čvora,
    Cvor *levo, *desno;                     // - levo i desno podstablo,
    Cvor(int b, Cvor* l=nullptr, Cvor* d=nullptr) // - konstruktor.
      : broj(b), levo(l), desno(d) {}
  };
  Cvor* stb;                                // Pokazivač na koren stabla.
 
public:
  Stablo(): stb(nullptr) {}                 // Prazno stablo.
  Stablo(int b): stb(new Cvor(b)) {}        // Konverzija.
  Stablo(const Stablo& s) { stb = kopiraj(s.stb); }    // Kopiranje.
  Stablo(Stablo&& s) { stb = s.stb; s.stb = nullptr; } // Premeštanje.
  ~Stablo() { brisi(stb); }                 // Destruktor.
  bool prazno() const { return stb == nullptr; } // Da li je stablo prazno?
  int vel() const { return vel(stb); }      // Broj čvorova u stablu.
  int zbir() const { return zbir(stb); }    // Zbir brojeva u stablu.
  void pisi_KLD() const { pisi_KLD(stb); }  // Prefiksno pisanje.
  void pisi_LKD() const { pisi_LKD(stb); }  // Infiksno pisanje.
  void pisi_LDK() const { pisi_LDK(stb); }  // Postfiksno pisanje.
  void crtaj() const { crtaj(stb, 0); }     // Grafički prikaz stabla.
  int pojav(int b) const                    // Broj pojavljivanja.
    { return pojav(stb, b); }
  int min() const {                         // Najmanji u stablu.
    if (!stb) exit(1);                      //   (ne sme biti prazno)
    return min(stb);
  }
  int max() const {                         // Najveći u stablu.
    if (!stb) exit(1);                      //   (ne sme biti prazno)
    return max(stb);
  }
  Cvor* nadji(int b) const                  // Traženje u stablu.
    { return nadji(stb, b); }
  void dodaj(int b) { dodaj(stb, b); }      // Dodavanje u stablo.
  void citaj(int n);                        // Čitanje stabla.
  void prazni() { brisi(stb); }             // Pražnjenje stabla.
  void izost(int b) { izost(stb, b); }      // Izostavljanje iz stabla.
  void balans() { balans(stb); }            // Balansiranje stabla.
private:                                    // POMOĆNE REKURZIVNE METODE:
  static Cvor* kopiraj(Cvor* stb) {         // Kopiranje stabla.
    return stb ?
      new Cvor(stb->broj, kopiraj(stb->levo), kopiraj(stb->desno)) : 0;
  }

  static int vel(Cvor* stb)                 // Broj čvorova u stablu.
    { return stb ? 1 + vel(stb->levo) + vel(stb->desno) : 0; }

  static int zbir(Cvor* stb)                // Zbir brojeva u stablu.
    { return stb ? stb->broj + zbir(stb->levo) + zbir(stb->desno) : 0; }

  static void pisi_KLD(Cvor* stb) {         // Prefiksno ispisivanje.
    if (stb) {
      cout << stb->broj << ' '; pisi_KLD(stb->levo); pisi_KLD(stb->desno);
    }
  }

  static void pisi_LKD(Cvor* stb) {         // Infiksno ispisivanje.
    if (stb) {
      pisi_LKD(stb->levo); cout << stb->broj << ' '; pisi_LKD(stb->desno);
    }
  }

  static void pisi_LDK(Cvor* stb) {         // Postfiksno ispisivanje.
    if (stb) {
      pisi_LDK(stb->levo); pisi_LDK(stb->desno); cout << stb->broj << ' ';
    }
  }
 
  static void crtaj(Cvor* stb, int nivo) {  // Grafički prikaz stabla.
    if (stb) {
      crtaj(stb->desno, nivo+1);
      cout << setw(4*nivo) << "" << stb->broj << endl;
      crtaj(stb->levo,  nivo+1);
    }
  }

  static int pojav(Cvor* stb, int b) {   // Broj pojavljivanja
    return stb ? (stb->broj==b)+pojav(stb->levo,b) // u stablu
                               +pojav(stb->desno,b)
               : 0;
  }

  static int min(Cvor* stb)                 // Najmanji u stablu.
    { return stb->levo ? min(stb->levo) : stb->broj; }

  static int max(Cvor* stb)                 // Najveći u stablu.
    { return stb->desno ? max(stb->desno) : stb->broj; }

  static Cvor* nadji(Cvor* stb, int b){     // Traženje u stablu.
    if (!stb)           return nullptr;
    if (stb->broj == b) return stb;
    if (stb->broj >  b) return nadji(stb->levo,  b);
                        return nadji(stb->desno, b);
  }

  static void dodaj(Cvor*& stb, int b) {    // Dodavanje u stablo.
    if (!stb)               stb = new Cvor(b);
    else if (stb->broj > b) dodaj(stb->levo,  b);
    else if (stb->broj < b) dodaj(stb->desno, b);
    else if (rand()/(RAND_MAX+1.) < 0.5)
                            dodaj(stb->levo,  b);
    else                    dodaj(stb->desno, b);
  }



  static void brisi(Cvor*& stb) {           // Oslobađanje memorije.
    if (stb) {
      brisi(stb->levo); brisi(stb->desno); delete stb; stb = nullptr;
    }
  }

  static void izost(Cvor*& stb, int b) {    // Izostavljanje iz stabla.
    if (stb) {
      if      (stb->broj > b) izost(stb->levo,  b);
      else if (stb->broj < b) izost(stb->desno, b);
      else if (stb->levo) {
        int m = max(stb->levo);
        stb->broj = m; izost(stb->levo,  m);
      } else if (stb->desno) {
        int m = min(stb->desno);
        stb->broj = m; izost(stb->desno, m);
      } else {
        delete stb; stb = nullptr;
      }
    }
  }

  static void balans(Cvor*& stb);           // Balansiranje stabla.
};
